// @ts-nocheck

$(function () {

  // An overlay div that is shown when request submission is in progress.
  // There does not seem to be a pure CSS solution to keep the size of the overlay div in sync with the size the section div,
  // therefore we catch the events that change the size of the section div and adjust the size of the overlay div dynamically.
  $('#contactsupport-section-div').prepend('<div id="contactsupport-section-cover" class="section-cover displaynone"></div>');

  function adjustOverlaySize() {
    $('#contactsupport-section-cover').width($('#contactsupport-section-div').outerWidth());
    $('#contactsupport-section-cover').height($('#contactsupport-section-div').outerHeight());
  }

  $(window).on('resize', adjustOverlaySize);

  $('#support-enable-trace-log').on('click', () => {
    vscode.postMessage({ handlerId: 'contactsupport', type: 'enableTraceLog' });
    return false;
  });

  $('#contact-support-add-logs').on('change', function () {
    if ($('#contact-support-add-logs').is(':checked')) {
      $('#contact-support-logs-table-wrapper').removeClass('displaynone');
      $('#contact-support-add-other-files').removeClass('displaynone');
    }
    else {
      $('#contact-support-logs-table-wrapper').addClass('displaynone');
      $('#contact-support-add-other-files').addClass('displaynone');
    }
  });

  $('#contact-support-add-other-files').on('click', () => {
    vscode.postMessage({ handlerId: 'contactsupport', type: 'addFile' });
  });

  $('#contact-support-cancel').on('click', () => {
    vscode.postMessage({ handlerId: 'contactsupport', type: 'cancel' });
  });

  $('#contact-support-submit').on('click', () => {
    adjustOverlaySize();
    $('#contactsupport-section-cover').removeClass('displaynone');
    vscode.postMessage({ handlerId: 'contactsupport', type: 'submit', description: $('#support-description').val(), includeLogs: $('#contact-support-add-logs').is(':checked') });
  });

  $(window).on('message', $e => {
    const event = $e.originalEvent;
    const message = event.data; // The json data that the extension sent
    switch (message.type) {
      case 'fillDescription':
        $('#support-description').val(message.description);
        break;
      case 'fillLogFilesTable':
        const logsTableBody = $("#logs-table-body");
        logsTableBody.find(".cloned").remove();
        for (let logFileData of message.logFiles) {
          const logRow = $('#--template-log-row').clone();
          logRow.html(logRow.html()
            .replace('--TEMPLATE-LOG-FILE-NAME', logFileData.baseName)
            .replace('--TEMPLATE-LOG-FILE-PATH', logFileData.fullName)
            .replace('--TEMPLATE-LOG-FILE-SIZE', logFileData.size)
            .replace('--template-remove-log-button-id',));
          logRow.css('display', 'table-row');
          logRow.addClass("cloned");
          logsTableBody.append(logRow);
          logRow.find("a").on('click', () => {
            vscode.postMessage({ handlerId: 'contactsupport', type: 'openFile', path: logFileData.fullName });
            return false;
          });
          logRow.find("button").on('click', () => {
            vscode.postMessage({ handlerId: 'contactsupport', type: 'removeFile', path: logFileData.fullName });
          });
          const removeBtn = logRow.find("button");
          if (logFileData.userAdded) {
            removeBtn.removeClass('displaynone');
          }
          else {
            removeBtn.addClass('displaynone');
          }
        }
        break;
      case 'showTraceLevelLogWarning':
        if (message.visible) {
          $('#contact-support-header-msg').removeClass('displaynone');
        }
        else {
          $('#contact-support-header-msg').addClass('displaynone');
        }
        $('#contact-support-current-log-level').html(message.level);
        break;
      case 'enableSupportForm':
        $('#contactsupport-section-cover').addClass('displaynone');
        break;
    }
  });

  vscode.postMessage({ handlerId: 'contactsupport', type: 'contactsupportShown' });

});