// @ts-nocheck

$(function () {
  $('.guidestep-expand-btn').on('click', e => {
    const btn = e.currentTarget;
    vscode.postMessage({ handlerId: 'setupguide', type: 'stepExpandCollapse', step: $(btn).attr('aria-step-id') });
  });

  $('.guidestep-mark-btn').on('click', e => {
    const btn = e.currentTarget;
    vscode.postMessage({ handlerId: 'setupguide', type: 'stepMarkCompleted', step: $(btn).attr('aria-step-id') });
  });

  $('#setupguide-to-instructions').on('click', e => {
    $('#setupguide-completed').addClass('displaynone');
    $('#setupguide-steps').removeClass('displaynone');
    return false;
  });

  $('#dont-show-setup-guide').on('click', e => {
    vscode.postMessage({ handlerId: 'setupguide', type: 'dontShowSetupGuide' });
    return false;
  });

  function updateMarkButton(buttonMark, flags) {
    if (flags.locked) {
      buttonMark.prop('disabled', true);
      buttonMark.find('.lock').removeClass('displaynone');
      buttonMark.find('.checkmark').addClass('displaynone');
      buttonMark.attr('title', 'Complete the previous step to continue');
    }
    else {
      buttonMark.find('.lock').addClass('displaynone');
      buttonMark.find('.checkmark').removeClass('displaynone');
      if (flags.marked) {
        buttonMark.addClass('marked');
        buttonMark.prop('disabled', true);
        buttonMark.attr('title', 'Completed');
      }
      else {
        buttonMark.removeClass('marked');
        buttonMark.prop('disabled', !flags.canMark);
        if (flags.canMark) {
          buttonMark.attr('title', 'Mark the step as complete');
        }
        else {
          buttonMark.attr('title', buttonMark.attr('aria-step-id') === 'RELOAD' ?
            'The step will be marked complete on reload detection' :
            'The step will be marked complete on license activation');
        }
      }
    }
  }

  function updateStepDiv(step) {
    const stepDiv = $(`#guidestep-${step.step}`);
    stepDiv.removeClass('displaynone');
    const buttonExpand = stepDiv.find('.guidestep-expand-btn');
    buttonExpand.attr('aria-step-id', step.step);
    buttonExpand.prop('disabled', false);
    const buttonMark = stepDiv.find('.guidestep-mark-btn');
    buttonMark.attr('aria-step-id', step.step);
    buttonMark.removeClass('displaynone');
    const errMsgDiv = stepDiv.find('.info-message');
    errMsgDiv.addClass('displaynone');
    const errIcon = stepDiv.find('.error-icon');
    errIcon.addClass('displaynone');
    switch (step.state) {
      case 'LOCKED':
        updateMarkButton(buttonMark, { locked: true });
        break;
      case 'AVAILABLE':
        updateMarkButton(buttonMark, { locked: false, marked: false, canMark: step.canMarkAsComplete });
        break;
      case 'DONE':
        updateMarkButton(buttonMark, { locked: false, marked: true });
        break;
      case 'ERROR':
        updateMarkButton(buttonMark, { locked: false, marked: false, canMark: false });
        errMsgDiv.removeClass('displaynone');
        errMsgDiv.html(step.errorText ?? 'Unknown error.');
        buttonMark.addClass('displaynone');
        errIcon.removeClass('displaynone');
        break;
    }
    stepDiv.find('.guidestep-content').toggleClass('displaynone', !step.isOpen);
    buttonExpand.toggleClass('expanded', step.isOpen);
  }

  $(window).on('message', $e => {
    const event = $e.originalEvent;
    switch (event.data.type) {
      case 'updateGuideSteps':
        $('.guidestep').addClass('displaynone');
        const steps = event.data.steps;
        for (step of steps) {
          updateStepDiv(step);
        }
        break;
      case 'showCompleted':
        $('#setupguide-completed').removeClass('displaynone');
        $('#setupguide-steps').addClass('displaynone');
        break;
    }
  });

  vscode.postMessage({ handlerId: 'setupguide', type: 'setupguideShown' });
});