// @ts-nocheck

$(function () {
    const vscode = acquireVsCodeApi();

    $('a.rootlevel').click(e => {
        changeSelection($(e.currentTarget));
    });

    function changeSelection(selectedAnchor) {
        if (selectedAnchor.hasClass('selected')) {
            return;
        }

        removeAllMenuSelections();
        setItemSelection(selectedAnchor, true);
    }

    function removeAllMenuSelections() {
        $('a.rootlevel').each((_, anchor) => {
            setItemSelection($(anchor), false);
        });
    }

    function setItemSelection(anchor, selected) {
        if (selected) {
            anchor.addClass('selected');
        }
        else {
            anchor.removeClass('selected');
        }
        if (anchor.hasClass('expandable')) {
            const menuDiv = $('#' + anchor.prop('id') + '-menu');
            if (selected) {
                anchor.addClass('expanded');
                menuDiv.css('maxHeight', menuDiv.prop('scrollHeight') + 'px');
            }
            else {
                anchor.removeClass('expanded');
                menuDiv.css('maxHeight', '');
            }
        }
    }

    $('a').click(e => {
        notifyItemSelected(e.currentTarget);
    });

    function notifyItemSelected(anchor) {
        vscode.postMessage({ type: 'menuSelected', menuId: anchor.id });
    }

    $('#go-to-projects').click(goToProjects);

    function goToProjects() {
        vscode.postMessage({ type: 'goToProjects' });
    }

    $(window).on('message', $e => {
        const event = $e.originalEvent;
        switch (event.data.type) {
            case 'panelClosed':
                removeAllMenuSelections();
                break;
            case 'selectPage':
                {
                    removeAllMenuSelections();
                    let anchor = $('#' + event.data.menuId);
                    setItemSelection(anchor, true);
                }
                break;
            case 'ui.updateLicenseData':
                {
                    if (event.data.isValidLicense) {
                        $("#user-settings-email").removeClass("displaynone");
                        $("#user-settings-missing-license-warning").addClass("displaynone");
                        if (event.data.email) {
                            $("#user-settings-email").html(event.data.email);
                        } else {
                            $("#user-settings-email").html("");
                        }
                    } else {
                        $("#user-settings-email").addClass("displaynone");
                        $("#user-settings-missing-license-warning").removeClass("displaynone");
                    }
                }
                break;
        }
    });

    vscode.postMessage({ type: 'sidebarLoaded' }); // notifies extension that sidebar got loaded.

});