// @ts-nocheck

class TimeDropdown {
  #rootDivId;
  #minsIdPrefix;
  #secsIdPrefix;
  #jqButton;
  #jqDropdownDiv;
  #jqMinutesCol;
  #jqSecondsCol;

  timeSavedValueMinutes;
  timeSavedValueSeconds;

  constructor(rootDivId, updateStatsFn) {
    this.#rootDivId = rootDivId;
    this.#minsIdPrefix = rootDivId + "-radio-min";
    this.#secsIdPrefix = rootDivId + "-radio-sec";

    this.#jqButton = $(`#${this.#rootDivId} button`);
    this.#jqDropdownDiv = $(`#${this.#rootDivId} .dropdown-content`);
    this.#jqMinutesCol = $(`#${this.#rootDivId} .dropdown-content-col:first-of-type`);
    this.#jqSecondsCol = $(`#${this.#rootDivId} .dropdown-content-col:last-of-type`);

    for (let i = 0; i < 30; i++) {
      let displVal = ("0" + i).substr(-2);
      let radioId = this.#minsIdPrefix + i;
      this.#jqMinutesCol.append(`<div>
        <input type="radio" name="minutes" id="${radioId}" value="${i}" />
        <label for="${radioId}" tabindex="0">${displVal}</label>
        </div>`);
    }
    for (let i = 0; i < 60; i += 2) {
      let displVal = ("0" + i).substr(-2);
      let radioId = this.#secsIdPrefix + i;
      this.#jqSecondsCol.append(`<div>
          <input type="radio" name="seconds" id="${radioId}" value="${i}" />
          <label for="${radioId}" tabindex="0">${displVal}</label>
          </div>`);
    }

    this.#jqButton.on("click", e => {
      this.#jqDropdownDiv.toggleClass('dropdown-showing');
      if (this.#jqDropdownDiv.hasClass('dropdown-showing')) {
        this.#jqDropdownDiv.width(this.#jqButton.outerWidth());
        let id = this.#jqMinutesCol.find('input[name="minutes"]:checked').attr('id');
        $(`label[for="${id}"]`).focus();
        const labelHeight = this.#jqMinutesCol.find('label:first-of-type').height();
        this.#jqMinutesCol[0].scroll(0, this.timeSavedValueMinutes * labelHeight - 2 * labelHeight);
        this.#jqSecondsCol[0].scroll(0, this.timeSavedValueSeconds / 2 * labelHeight - 2 * labelHeight);
      }
      else {
        updateStatsFn();
      }
    });

    $(document).on('click', e => {
      if (!$(`#${this.#rootDivId}`).find(e.target).length) {
        this.#jqDropdownDiv.removeClass('dropdown-showing');
        updateStatsFn();
      }
    });

    $(`#${this.#rootDivId} .dropdown-content-col`).on('wheel', e => {
      e.currentTarget.scrollBy(0, e.originalEvent.deltaY);
      return false;
    });

    $(`#${this.#rootDivId} .dropdown-content-col label`).on('keydown', e => {
      console.log("keyCode=", e.keyCode);
      let name = $(e.currentTarget).attr('for');
      let prefix = name.substr(0, this.#minsIdPrefix.length); //assume mins and secs ID prefixes have the same length
      let index = parseInt(name.substr(this.#minsIdPrefix.length));
      let interval = prefix === this.#minsIdPrefix ? 1 : 2;
      let id = undefined;
      if (e.keyCode === 38) { //up
        let previndex = index - interval;
        if (previndex >= 0) {
          id = prefix + previndex;
        }
      }
      else if (e.keyCode === 40) { //down
        let nextindex = index + interval;
        if (nextindex < (prefix === 'm' ? 30 : 60)) {
          id = prefix + nextindex;
        }
      }
      else if (e.keyCode === 39) { //right
        id = $(`#${this.#rootDivId} input[name="seconds"]:checked`).attr('id');
      }
      else if (e.keyCode === 37) { //left
        id = $(`#${this.#rootDivId} input[name="minutes"]:checked`).attr('id');
      }
      else if (e.keyCode === 9) { //tab
        if (prefix === this.#minsIdPrefix) {
          id = $(`#${this.#rootDivId} input[name="seconds"]:checked`).attr('id');
        }
        else {
          id = $(`#${this.#rootDivId}  input[name="minutes"]:checked`).attr('id');
        }
      }
      else if (e.keyCode === 13) { //enter
        this.#jqDropdownDiv.removeClass('dropdown-showing');
        updateStatsFn();
        this.#jqButton.focus();
        return false;
      }
      if (id) {
        let target = $(`#${id}`);
        target.click();
        $(`label[for="${id}"]`).focus();
        return false;
      }
    });

    $(`#${this.#rootDivId} input[type='radio']`).on('change', e => {
      this.timeSavedValueMinutes = parseInt($(`#${this.#rootDivId} input[name="minutes"]:checked`).val() || "0");
      this.timeSavedValueSeconds = parseInt($(`#${this.#rootDivId} input[name="seconds"]:checked`).val() || "0");
      this.updateDeploymentTimeButtonText();
    });

  }

  updateDeploymentTimeButtonText() {
    const dispMinutes = ("0" + this.timeSavedValueMinutes).substr(-2);
    const dispSeconds = ("0" + this.timeSavedValueSeconds).substr(-2);
    this.#jqButton.find('span').text(`${dispMinutes}:${dispSeconds} (mm:ss)`);
  }

  setDurationValue(secs) {
    this.timeSavedValueMinutes = Math.min(Math.floor(secs / 60), 29);
    this.timeSavedValueSeconds = secs % 60 - secs % 2;
    this.updateDeploymentTimeButtonText();

    $(`#${this.#minsIdPrefix}${this.timeSavedValueMinutes}`).prop('checked', true);
    $(`#${this.#secsIdPrefix}${this.timeSavedValueSeconds}`).prop('checked', true);
  }

}

$(function () {

  const timeDropdown = new TimeDropdown('deployment-time-dropdown', updateStats);

  function updateStats() {
    const deploymentTime = timeDropdown.timeSavedValueMinutes * 60 + timeDropdown.timeSavedValueSeconds;
    const periodLength = parseInt($('#time-saved-period>option:selected').val());
    vscode.postMessage({ handlerId: 'timesavedcalculator', type: 'loadStats', deployTime: deploymentTime, periodLength: periodLength });
  }

  $('#time-saved-period').on('change', updateStats);

  $(window).on('message', $e => {
    const event = $e.originalEvent;
    switch (event.data.type) {
      case 'updateTimeSavedCalculatorStats':
        const stats = event.data.stats;

        const secs = parseInt(stats.avgRedeployTime);
        timeDropdown.setDurationValue(secs);

        $('#time-saved-redeploys').text(stats.redeployCount);
        $('#time-saved-hours').text(stats.timeSavedHours);
        break;
    }
  });

  vscode.postMessage({ handlerId: 'timesavedcalculator', type: 'loadStats', deployTime: 0, periodLength: 30 });
});
