// taken from https://github.com/Auroratide/toggle-switch - uses ISC License

const template = document.createElement('template');
template.innerHTML = `
  <style>
    :host {
      display: inline-block;
      width: 32px;
      height: 16px;
      cursor: pointer;
    }

    span {
      box-sizing: border-box;
      display: inline-block;
      line-height: 1;
    }

    [part="track"] {
      width: 100%;
      height: 100%;
      background-color: #404A53;
      text-align: left;
      border-radius: 8px;
      border: 1px solid #404A53;
    }
    
    [part="track"]:hover {
      background-color: #8997A4;
      border-color: #8997A4;
    }

    [part="slider"] {
      margin: 1px;
      width: 12px;
      height: 12px;
      background-color: #FFFFFF;
      vertical-align: text-top;
      border-radius: 6px;
    }

    [part="track"], [part="slider"] {
      transition: all 256ms;
    }

    :host([checked]) [part="slider"] {
      transform: translateX(16px);
    }

    :host([checked]) [part="track"] {
      background-color: #6C9C23;
      border-color: #6C9C23;
    }

    :host([checked]) [part="track"]:hover {
      background-color: #598A1B;
      border-color: #598A1B;
    }

    :host([disabled]) {
      cursor: default;
    }

    :host([disabled]) [part="track"],
    :host([disabled]) [part="track"]:hover {
      background-color: #24292E;
      border-color: #384149;
    }

    :host([disabled]) [part="slider"] {
      background-color: #A6B0BA;
    }

  </style>

  <span part="track">
    <span part="slider"></span>
  </span>
`;

class ToggleSwitch extends HTMLElement {
  static elementName = 'toggle-switch';

  static get observedAttributes() {
    return ['checked', 'disabled'];
  }

  constructor() {
    super();

    this
      .attachShadow({ mode: 'open' })
      .appendChild(template.content.cloneNode(true));
  }

  connectedCallback() {
    this.setAttribute('role', 'switch');
    this.setAttribute('tabindex', '0');
    this.addEventListener('click', this.toggle);
  }

  disconnectedCallback() {
    this.removeEventListener('click', this.toggle);
  }

  attributeChangedCallback(name, oldValue, newValue) {
    if (name === 'checked') {
      this.setAttribute('aria-checked', this.checked.toString());
      this.dispatchEvent(new CustomEvent('toggle-switch:change', {
        detail: {
          checked: this.checked
        }
      }));
    }
  }

  get checked() {
    return this.hasAttribute('checked');
  }
  set checked(value) {
    this.toggleAttribute('checked', value);
  }

  get disabled() {
    return this.hasAttribute('disabled');
  }
  set disabled(value) {
    this.toggleAttribute('disabled', value);
  }

  toggle = () => {
    if (!this.disabled) {
      this.checked = !this.checked;
    }
  };
}

window.customElements.define(ToggleSwitch.elementName, ToggleSwitch);