//@ts-check

console.log('====== Uninstall callback of JRebel extension starting! ======');

const VM_ARGS_KEY = 'args.vm.override.string';
const REBELARGS = "-DREBELARGS";

try {
  const fs = require('fs');
  const path = require('path');

  if (process.env.HOME) {
    const rspPath = path.join(process.env.HOME, '.rsp');
    if (fs.existsSync(rspPath)) {
      console.log('Found ' + rspPath);
      for (let dirEnt of fs.readdirSync(rspPath, { withFileTypes: true })) {
        if (dirEnt.isDirectory()) {
          console.log('Found ' + dirEnt.name);
          const serversPath = path.join(rspPath, dirEnt.name, 'servers');
          if (fs.existsSync(serversPath)) {
            console.log('Found ' + serversPath);
            for (let serverFileEnt of fs.readdirSync(serversPath, { withFileTypes: true })) {
              if (serverFileEnt.isFile()) {
                try {
                  console.log('Found ' + serverFileEnt.name);
                  const serverFilePath = path.join(serversPath, serverFileEnt.name);
                  const content = fs.readFileSync(serverFilePath);
                  const conf = JSON.parse(content.toString());
                  let vmArgsValue = conf[VM_ARGS_KEY];
                  if (vmArgsValue) {
                    console.log('VM arguments: ' + vmArgsValue);
                    vmArgsValue = removeAgentpath(vmArgsValue);
                    vmArgsValue = removeArgsfile(vmArgsValue);
                    vmArgsValue = removeRebelNotificationUrl(vmArgsValue);
                    vmArgsValue = removeMiscRebelProperties(vmArgsValue);
                    if (vmArgsValue !== conf[VM_ARGS_KEY]) {
                      console.log('VM arguments modified: ' + vmArgsValue);
                      conf[VM_ARGS_KEY] = vmArgsValue;
                      const newContent = JSON.stringify(conf, undefined, '  ');
                      const backupDirPath = path.join(process.env.HOME, '.jrebel', 'vscode-server-conf-backups');
                      if (!fs.existsSync(backupDirPath)) {
                        fs.mkdirSync(backupDirPath, { recursive: true });
                      }
                      const serverFileBackupPath = path.join(backupDirPath, serverFileEnt.name);
                      fs.copyFileSync(serverFilePath, serverFileBackupPath);
                      console.log('Backed up ' + serverFilePath + ' to ' + serverFileBackupPath);
                      fs.writeFileSync(serverFilePath, newContent);
                      console.log("Updated " + serverFilePath);
                    }
                    else {
                      console.log('VM arguments string does not contain any JRebel-specific arguments');
                    }
                  }
                  else {
                    console.log('VM arguments string is not present');
                  }
                }
                catch (e1) {
                  console.error(e1);
                }
              }
            }
          }
          else {
            console.log('Not found ' + serversPath);
          }
        }
      }
    }
    else {
      console.log('Not found ' + rspPath);
    }
  }
}
catch (e) {
  console.error(e);
}


/**
 * @param {string} currentValue
 */
function removeAgentpath(currentValue) {
  if (currentValue.includes(REBELARGS)) {
    return currentValue.replace(REBELARGS, "").trim();
  }

  const agentMatcher = /-agentpath:.*jrebel(64|32)\.(dll|so|dylib)/;
  if (currentValue.match(agentMatcher)) {
    return currentValue.replace(agentMatcher, "").trim();
  }
  return currentValue;
}

/**
 * @param {string} currentValue
 */
function removeArgsfile(currentValue) {
  const agentMatcher = /@.*vscodejvmargsfile\.txt/;
  if (currentValue.match(agentMatcher)) {
    return currentValue.replace(agentMatcher, "").trim();
  }
  return currentValue;
}

/**
 * @param {string} val
 */
function removeRebelNotificationUrl(val) {
  // FIXME JR-11684 Use 127.0.0.2 address
  const m = /-Drebel\.notification\.url=http:\/\/127\.0\.0\.1:(\d+)/;
  if (val.match(m)) {
    return val.replace(m, '').trim();
  }
  return val;
}

/**
 * @param {string} val
 */
function removeMiscRebelProperties(val) {
  let result = val;
  result = result.replace(/-Drebel\.base=\S+/, '');
  result = result.replace(/-Drebel\.env\.ide\.plugin\.version=\S+/, '');
  result = result.replace(/-Drebel\.env\.ide\.version=\S+/, '');
  result = result.replace(/-Drebel\.env\.ide\.product=\S+/, '');
  result = result.replace(/-Drebel\.env\.ide=\S+/, '');
  result = result.trim();
  return result;
}

console.log('====== Uninstall callback of JRebel extension completed! ======');